/*
 *  clueless-kmean is a variant of k-mean which enforces balanced
 *  distribution of classes in every cluster
 *
 *  Copyright (c) 2013 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of clueless-kmean.
 *
 *  clueless-kmean is free software: you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  version 3 as published by the Free Software Foundation.
 *
 *  clueless-kmean is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with selector.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef ARRAYS_H
#define ARRAYS_H

template<class T>
T inline sqdist(int dim, T *x, T *y) {
  T result = 0;
  for(int k = 0; k < dim; k++) result += sq(x[k] - y[k]);
  return result;
}

template<class T>
T inline sum(int dim, T *x) {
  T result = 0;
  for(int k = 0; k < dim; k++) result += x[k];
  return result;
}

template<class T>
T inline dotprod(int dim, T *x, T *y) {
  T result = 0;
  for(int k = 0; k < dim; k++) result += x[k] * y[k];
  return result;
}

template<class T>
void fill_vector(int dim, T *vector, T v) {
  for(int k = 0; k < dim; k++) {
    vector[k] = v;
  }
}

template<class T>
void copy_vector(int dim, T *vector_dst, T *vector_src) {
  for(int k = 0; k < dim; k++) {
    vector_dst[k] = vector_src[k];
  }
}

template<class T>
T **allocate_array(int a, int b) {
  T *whole = new T[a * b];
  T **array = new T *[a];
  for(int k = 0; k < a; k++) {
    array[k] = whole;
    whole += b;
  }
  return array;
}

template<class T>
void deallocate_array(T **array) {
  if(array) {
    delete[] array[0];
    delete[] array;
  }
}

template<class T>
void fill_array(int a, int b, T **array, T v) {
  for(int k = 0; k < a * b; k++) {
    array[0][k] = v;
  }
}

template<class T>
T ***allocate_volume(int a, int b, int c) {
  T *whole = new T[a * b * c];
  T **column = new T *[a * b];
  T ***volume = new T **[a];

  for(int k = 0; k < a; k++) {
    volume[k] = column;
    for(int l = 0; l < b; l++) {
      column[l] = whole;
      whole += c;
    }
    column += b;
  }

  return volume;
}

template<class T>
void deallocate_volume(T ***volume) {
  if(volume) {
    delete[] volume[0][0];
    delete[] volume[0];
    delete[] volume;
  }
}

template<class T>
void fill_volume(int a, int b, int c, T ***volume, T v) {
  for(int k = 0; k < a * b * c; k++) {
    volume[0][0][k] = v;
  }
}

#endif
